# -*- coding: utf-8 -*-
# ++ This file `lru_wrapped.py` is generated at 4/4/16 4:53 PM ++
from django.utils import lru_cache

from .utils import get_group_key as utils_get_group_key
from .utils import get_user_key as utils_get_user_key
from .utils import get_site_urlconf as utils_get_site_urlconf
from .utils import get_generated_urlconf_file as utils_get_generated_urlconf_file
from .utils import get_installed_apps_urlconf as utils_get_installed_apps_urlconf
from .utils import get_generated_urlconf_module as utils_get_generated_urlconf_module

__author__ = "Md Nazrul Islam<connect2nazrul@gmail.com>"

""" All lru cached functions will put here, the purpose is here, we will have non-cached version as well"""


@lru_cache.lru_cache(maxsize=None)
def get_site_urlconf(site):
    return utils_get_site_urlconf(site)


@lru_cache.lru_cache(maxsize=None)
def get_group_key(request, group, prefix='hacl', suffix=None):
    return utils_get_group_key(request, group, prefix, suffix)


@lru_cache.lru_cache(maxsize=None)
def get_user_key(request, prefix='hacl', suffix=None):
    return utils_get_user_key(request, prefix, suffix)


@lru_cache.lru_cache(maxsize=None)
def get_generated_urlconf_file(route_name, prefix=None):
    return utils_get_generated_urlconf_file(route_name, prefix)


@lru_cache.lru_cache(maxsize=None)
def get_generated_urlconf_module(filename, validation=True):
    return utils_get_generated_urlconf_module(filename, validation)


@lru_cache.lru_cache(maxsize=None)
def get_installed_apps_urlconf(pattern=r'*urls.py', to_json=False, exclude=()):
    return utils_get_installed_apps_urlconf(pattern, to_json, exclude)


def clean_all_lru_caches():
    """
    :return:
    """
    get_site_urlconf.cache_clear()
    get_group_key.cache_clear()
    get_user_key.cache_clear()
    get_generated_urlconf_module.cache_clear()
    get_generated_urlconf_module.cache_clear()
    get_installed_apps_urlconf.cache_clear()



# -*- coding: utf-8 -*-
# ++ This file `__init__.py.py` is generated at 4/1/16 6:07 PM ++
from django.views.generic import TemplateView

__author__ = "Md Nazrul Islam<connect2nazrul@gmail.com>"


class IndexView(TemplateView):
    pass


class AboutView(TemplateView):
    pass



# -*- coding: utf-8 -*-
# ++ This file `init_hacs.py` is generated at 5/16/16 1:16 PM ++

from django.conf import settings
from django.contrib.sites.models import Site
from django.core.management.base import BaseCommand

from hacs.models import RoutingTable
from hacs.models import SiteRoutingTable

__author__ = "Md Nazrul Islam<connect2nazrul@gmail.com>"


class Command(BaseCommand):
    """
    """
    help = "HACS: Command tool for initialize hacs environment"
    can_import_settings = True

    def add_arguments(self, parser):
        """
        :param parser:
        :return:
        """
        parser.add_argument(
            '-d',
            '--domain-name',
            action='store',
            dest='domain_name',
            help='Default domain name for site'
        )
        parser.add_argument(
            '-u',
            '--urlconf-module',
            action='store',
            dest='urlconf_module',
            help='urlconf module name, that will used at default route'
        )
        parser.add_argument(
            '-r',
            '--route-name',
            action='store',
            dest='route_name',
            default='default-route',
            help='Default route name'

        )

    def handle(self, *args, **options):
        """
        :param args:
        :param options:
        :return:
        """
        domain = options['domain_name']
        while True:
            if not domain:
                domain = raw_input('Please provide domain name for site. [localhost]')
                if not domain:
                    domain = 'localhost'
            else:
                break

        urlconf = options['urlconf_module']
        while True:
            if not urlconf:
                urlconf = raw_input('Please provide root urlconf [%s]' % settings.ROOT_URLCONF)
                if not urlconf:
                    urlconf = settings.ROOT_URLCONF
            else:
                break

        try:
            site = Site.objects.get(domain=domain)
        except Site.DoesNotExist:
            site = Site(domain=domain, name=domain)
            site.save()

        try:
            route = RoutingTable.objects.get_by_natural_key(options['route_name'])
        except RoutingTable.DoesNotExist:
            route = RoutingTable(
                route_name=options['route_name'],
                urls=[
                    {
                        "prefix": None,
                        "url_module": urlconf,
                        "namespace": None,
                        "children": []
                    },
                ],
                handlers={
                    "handler400": "hacs.views.errors.bad_request",
                    "handler403": "hacs.views.errors.permission_denied",
                    "handler404": "hacs.views.errors.page_not_found",
                    "handler500": "hacs.views.errors.server_error"
                }
            )
            route.save()

        SiteRoutingTable(site=site, route=route).save()
        self.stdout.write('>>> Django Hybrid Access Control System is initialized and ready to use!')





from __future__ import unicode_literals

# ++ This file `fields.py` is generated at 3/4/16 5:54 PM ++
import ast
import json
from django.db import models
from django.utils import six
from django.utils.encoding import smart_text
from django.utils.translation import ugettext_lazy as _
from django.core.serializers.json import DjangoJSONEncoder

__author__ = "Md Nazrul Islam<connect2nazrul@gmail.com>"


class JsonField(models.TextField):
    """
    Idea From: https://djangosnippets.org/snippets/1478/
    """

    def __init__(self, *args, **kwargs):
        """
        :param args:
        :param kwargs:
        :return:
        """
        self.deserialize = kwargs.pop('deserialize', False)

        super(JsonField, self).__init__(*args, **kwargs)

    def to_python(self, value):
        """
        :param value:
        :return:
        """
        if not self.deserialize:

            return super(JsonField, self).to_python(value)

        if value == "":
            return None

        try:
            if isinstance(value, six.text_type):
                value = json.loads(value)

            elif isinstance(value, six.string_types):
                value = json.loads(smart_text(value))

        except ValueError:
            try:
                # Fallback: django serializer may cast dict or list value as string
                value = ast.literal_eval(value)
            except ValueError:
                raise
        return value

    def get_prep_value(self, value):
        """
        :param value:
        :return:
        """
        if value in('', "", True) or not value:
            return None

        value = super(JsonField, self).get_prep_value(value)
        if isinstance(value, (dict, list, set, tuple)):
            try:
                value = json.dumps(value, cls=DjangoJSONEncoder)
            except ValueError:
                raise
        return value

    def from_db_value(self, value, expression, connection, context):
        """
        :param value:
        :param expression:
        :param connection:
        :param context:
        :return:
        """
        return self.to_python(value)


class JsonDictField(JsonField):
    """
        Only allows python Dictionary data type data
    """

    def get_prep_value(self, value):

        if value in('', "", True) or not value:
            return None

        if not isinstance(value, dict):
            raise TypeError(_('Only `dict` type data is accepted but got data type `%s`' % type(value)))

        return super(JsonDictField, self).get_prep_value(value)


class JsonSequenceField(JsonField):
    """
        Only allows python list, set, tuple type data
    """

    def get_prep_value(self, value):

        if value in('', "", True) or not value:
            return None

        if not isinstance(value, (list, set, tuple)):
            raise TypeError(_('Only `list, set, tuple` type data is accepted but got data type `%s`' % type(value)))

        return super(JsonSequenceField, self).get_prep_value(value)


class DictField(JsonDictField):
    """
    """
    def __init__(self, *args, **kwargs):
        """
        :param args:
        :param kwargs:
        :return:
        """
        try:
            if not kwargs['deserialize']:
                kwargs['deserialize'] = True
        except KeyError:
            kwargs['deserialize'] = True

        super(DictField, self).__init__(*args, **kwargs)


class SequenceField(JsonSequenceField):
    """
    """
    def __init__(self, *args, **kwargs):
        """
        :param args:
        :param kwargs:
        :return:
        """
        try:
            if not kwargs['deserialize']:
                kwargs['deserialize'] = True
        except KeyError:
            kwargs['deserialize'] = True

        super(SequenceField, self).__init__(*args, **kwargs)


__all__ = ('JsonField', 'JsonDictField', 'JsonSequenceField', 'DictField', 'SequenceField')



# -*- coding: utf-8 -*-
# ++ This file `defaults.py` is generated at 3/3/16 6:15 AM ++
from django.apps import apps
from django.conf import settings
from django.utils._os import safe_join

from .globals import HACS_APP_NAME
__author__ = "Md Nazrul Islam<connect2nazrul@gmail.com>"

HACS_CACHE_SETTING_NAME = 'default'
HACS_FALLBACK_URLCONF = settings.ROOT_URLCONF
HACS_GENERATED_URLCONF_DIR = safe_join(apps.get_app_config(HACS_APP_NAME).path, 'generated')
HACS_SERIALIZED_ROUTING_DIR = None
HACS_USER_OBJECT_QUERY_CALLABLE = "hacs.utils.get_user_object"



# -*- coding: utf-8 -*-
import os
import sys
from django.conf import settings
from django.apps import AppConfig
from django.utils._os import safe_join
from django.utils.translation import ugettext_lazy as _

from .globals import HACS_APP_NAME
from .globals import HACS_APP_LABEL

__author__ = "Md Nazrul Islam<connect2nazrul@gmail.com>"


class HACSConfig(AppConfig):

    """
    """
    name = HACS_APP_NAME
    label = HACS_APP_LABEL
    verbose_name = _('Hybrid Access Control System')

    def ready(self):
        """
        :return:
        """
        _path = getattr(settings, 'HACS_GENERATED_URLCONF_DIR', safe_join(self.path, 'generated'))
        if not os.path.exists(_path):
            os.mkdir(_path)

        if _path not in sys.path:
            sys.path = sys.path[:] + [_path]

        return super(HACSConfig, self).ready()





from __future__ import unicode_literals

from django.db import models
from django.apps import apps
from django.utils import six
from django.utils import timezone
from django.contrib.sites.models import Site
from django.utils.translation import ugettext_lazy as _
from django.contrib.contenttypes.models import ContentType
from django.utils.encoding import python_2_unicode_compatible
from django.contrib.contenttypes.fields import GenericForeignKey


from .fields import DictField
from .fields import SequenceField

if not apps.is_installed('django.contrib.admin'):
    # Fallback LogEntry Model, if admin app not installed
    from django.contrib.admin.models import LogEntry as django_LogEntry

    class LogEntry(django_LogEntry):
        pass

__author__ = "Md Nazrul Islam<connect2nazrul@gmail.com>"


class RoutingTableManager(models.Manager):
    """"""
    use_in_migrations = True

    def get_by_natural_key(self, route_name):
        """
        :param route_name:
        :return:
        """
        return self.get(route_name=route_name)


@python_2_unicode_compatible
class RoutingTable(models.Model):
    """
    JSON Field Permitted Format/Python Data pattern
    -----------------------------------------------
    urls: [{'prefix': None, 'url_module': None, namespace=None, children: []}]
    OR [{'prefix': None, 'url_module': (module, app_name), namespace=None, children: []}]

    handlers: {'handler400': None, 'handler403': None, 'handler404': None, 'handler500': None}
    """
    route_name = models.SlugField(_('route name'),  null=False, blank=False, unique=True, db_index=True, max_length=127)
    description = models.TextField(_('description'), null=True, blank=True)
    urls = SequenceField(_('URLs'), null=False, blank=False, validators=[])
    handlers = DictField(_('Handlers'), null=True, blank=True, default='', validators=[])
    generated_module = models.TextField(_('Generated Module'), null=True, blank=True, default=None)
    is_active = models.BooleanField(_('Is Active'), null=False, blank=True, default=True)
    is_deleted = models.BooleanField(_('Soft Delete'), null=False, blank=True, default=False)
    created_on = models.DateTimeField(_('Created On'), blank=True, default=timezone.now)
    updated_on = models.DateTimeField(_('Last updated'), null=True, blank=True)

    objects = RoutingTableManager()

    class Meta:
        db_table = 'hacl_routing_table'
        verbose_name = _('routing table')
        verbose_name_plural = _('routes table')

    def natural_key(self):
        """
        :return:
        """
        return (self.route_name, )

    def __str__(self):
        """
        """
        return self.route_name


class SiteRoutingTableManager(models.Manager):
    """ """
    use_in_migrations = True

    def get_by_natural_key(self, site_natural_key):
        """
        :param site_natural_key:
        :return:
        """
        if isinstance(site_natural_key, six.string_types):
            site_natural_key = (site_natural_key,)

        try:
            if not isinstance(site_natural_key, (list, tuple,)):
                snk = (site_natural_key, )
            else:
                snk = site_natural_key
            site = Site.objects.db_manager(self.db).get_by_natural_key(*snk)
        except AttributeError:
            if isinstance(site_natural_key, six.integer_types):
                site = Site.objects.db_manager(self.db).get(pk=site_natural_key)
            else:
                raise

        return self.get(site=site)


class SiteRoutingTable(models.Model):

    """
    """
    route = models.ForeignKey(RoutingTable,
                              on_delete=models.CASCADE,
                              db_column='route_id',
                              db_constraint=False,
                              related_name='hacs_route_sites')
    site = models.OneToOneField(Site,
                             on_delete=models.CASCADE,
                             unique=True,
                             null=False,
                             blank=False,
                             related_name='hacs_site_routes')
    is_active = models.BooleanField(_('Is Active'), null=False, blank=True, default=True)
    created_on = models.DateTimeField(_('Created On'), blank=True, default=timezone.now)
    updated_on = models.DateTimeField(_('Last updated'), null=True, blank=True)

    objects = SiteRoutingTableManager()

    class Meta:
        db_table = 'hacs_sites_routing_table'
        verbose_name = _('site routing table')
        verbose_name_plural = _('sites routing table')

    def natural_key(self):
        """
        :return:
        """
        try:
            site_natural_key = self.site.natural_key()
        except AttributeError:
            # Right now `natural_key` is not implemented by django, but would be good if they do
            site_natural_key = self.site.pk

        return (site_natural_key, )
    natural_key.dependencies = ["hacs.RoutingTable", "sites.Site"]


class ContentTypeRoutingTableManager(models.Manager):
    """ """
    use_in_migrations = True

    def get_by_natural_key(self, site_nk, content_type_nk, object_id):
        """
        :param site_nk:
        :param content_type_nk
        :param object_id
        :return:
        """
        if isinstance(site_nk, six.string_types):
            site_nk = (site_nk,)

        if isinstance(content_type_nk, six.string_types):
            content_type_nk = (content_type_nk,)

        try:
            if not isinstance(site_nk, (list, tuple)):
                snk = (site_nk, )
            else:
                snk = site_nk
            site = Site.objects.db_manager(self.db).get_by_natural_key(*snk)
        except AttributeError:
            if isinstance(site_nk, six.integer_types):
                site = Site.objects.db_manager(self.db).get(pk=site_nk)
            else:
                raise

        return self.get(
            site=site,
            content_type=ContentType.objects.db_manager(self.db).get_by_natural_key(*content_type_nk),
            object_id=object_id
        )


class ContentTypeRoutingTable(models.Model):

    """
    """
    route = models.ForeignKey(RoutingTable,
                              on_delete=models.CASCADE,
                              db_column='route_id',
                              db_constraint=False,
                              related_name='hacs_route_contenttypes',
                              validators=[],
                              )
    site = models.ForeignKey(Site,
                             on_delete=models.CASCADE,
                             null=False,
                             blank=False,
                             related_name='hacs_site_contenttypes_at_routing_table'
                             )
    content_type = models.ForeignKey(ContentType, on_delete=models.CASCADE, validators=[])
    object_id = models.PositiveIntegerField()
    content_object = GenericForeignKey('content_type', 'object_id')
    is_active = models.BooleanField(_('Is Active'), null=False, blank=True, default=True)
    created_on = models.DateTimeField(_('Created On'),  blank=True, default=timezone.now)
    updated_on = models.DateTimeField(_('Last updated'), null=True, blank=True)

    objects = ContentTypeRoutingTableManager()

    class Meta:
        db_table = 'hacs_ct_routing_table'
        verbose_name = _('content type routing table')
        verbose_name_plural = _('content types routing table')
        unique_together = (("site", "content_type", "object_id"),)

    def natural_key(self):
        """
        :return:
        """
        try:
            site_natural_key = self.site.natural_key()
        except AttributeError:
            # Right now `natural_key` is not implemented by django, but would be good if they do
            site_natural_key = self.site.pk

        return (site_natural_key, self.content_type.natural_key(), self.object_id, )

    natural_key.dependencies = ["hacs.RoutingTable", "sites.Site", "contenttypes.ContentType"]



# -*- coding: utf-8 -*-
# ++ This file `globals.py` is generated at 3/3/16 6:07 AM ++

__author__ = "Md Nazrul Islam<connect2nazrul@gmail.com>"

HACS_APP_NAME = 'hacs'
HACS_APP_LABEL = HACS_APP_NAME
HACS_GENERATED_FILENAME_PREFIX = 'hacs__generated_'
HACS_SERIALIZED_ROUTE_DIR_NAME = 'hacs_routes'


class HACSSiteCache(object):
    """ Obviously this class is not tread safe, infact we don't need to be. """

    def __init__(self):

        """
        :return:
        """
        self.__storage__ = dict()

    def get(self, key, default=None):
        """
        :param key:
        :param default:
        :return:
        """
        return self.__storage__.get(key, default)

    def set(self, key, value):
        """
        :param key:
        :param value:
        :return:
        """
        return self.__storage__.update({key: value})

    def clear(self):
        """
        :return:
        """
        # Not sure it's make sense of performance optimized way
        del self.__storage__
        self.__storage__ = dict()

    def __getitem__(self, item):
        """
        :param item:
        :return:
        """
        return self.__storage__[item]

    def __setitem__(self, key, value):
        """
        :param key:
        :param value:
        :return:
        """
        self.__storage__[key] = value

    def __len__(self):
        """
        :return:
        """
        return len(self.__storage__)

    def __repr__(self):
        """
        :return:
        """
        return repr(self.__storage__)

    def __str__(self):
        """
        :return:
        """
        return str(self.__storage__)


HACS_SITE_CACHE = HACSSiteCache()



# -*- coding: utf-8 -*-
# Generated by Django 1.9.6 on 2016-05-24 17:52
from __future__ import unicode_literals

from django.db import migrations, models
import django.db.models.deletion
import django.utils.timezone
import hacs.fields
import hacs.models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('contenttypes', '0002_remove_content_type_name'),
        ('sites', '0002_alter_domain_unique'),
    ]

    operations = [
        migrations.CreateModel(
            name='ContentTypeRoutingTable',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('object_id', models.PositiveIntegerField()),
                ('is_active', models.BooleanField(default=True, verbose_name='Is Active')),
                ('created_on', models.DateTimeField(blank=True, default=django.utils.timezone.now, verbose_name='Created On')),
                ('updated_on', models.DateTimeField(blank=True, null=True, verbose_name='Last updated')),
                ('content_type', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='contenttypes.ContentType')),
            ],
            options={
                'db_table': 'hacs_ct_routing_table',
                'verbose_name': 'content type routing table',
                'verbose_name_plural': 'content types routing table',
            },
            managers=[
                ('objects', hacs.models.ContentTypeRoutingTableManager()),
            ],
        ),
        migrations.CreateModel(
            name='RoutingTable',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('route_name', models.SlugField(max_length=127, unique=True, verbose_name='route name')),
                ('description', models.TextField(blank=True, null=True, verbose_name='description')),
                ('urls', hacs.fields.SequenceField(verbose_name='URLs')),
                ('handlers', hacs.fields.DictField(blank=True, default='', null=True, verbose_name='Handlers')),
                ('generated_module', models.TextField(blank=True, default=None, null=True, verbose_name='Generated Module')),
                ('is_active', models.BooleanField(default=True, verbose_name='Is Active')),
                ('is_deleted', models.BooleanField(default=False, verbose_name='Soft Delete')),
                ('created_on', models.DateTimeField(blank=True, default=django.utils.timezone.now, verbose_name='Created On')),
                ('updated_on', models.DateTimeField(blank=True, null=True, verbose_name='Last updated')),
            ],
            options={
                'db_table': 'hacl_routing_table',
                'verbose_name': 'routing table',
                'verbose_name_plural': 'routes table',
            },
            managers=[
                ('objects', hacs.models.RoutingTableManager()),
            ],
        ),
        migrations.CreateModel(
            name='SiteRoutingTable',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('is_active', models.BooleanField(default=True, verbose_name='Is Active')),
                ('created_on', models.DateTimeField(blank=True, default=django.utils.timezone.now, verbose_name='Created On')),
                ('updated_on', models.DateTimeField(blank=True, null=True, verbose_name='Last updated')),
                ('route', models.ForeignKey(db_column='route_id', db_constraint=False, on_delete=django.db.models.deletion.CASCADE, related_name='hacs_route_sites', to='hacs.RoutingTable')),
                ('site', models.OneToOneField(on_delete=django.db.models.deletion.CASCADE, related_name='hacs_site_routes', to='sites.Site')),
            ],
            options={
                'db_table': 'hacs_sites_routing_table',
                'verbose_name': 'site routing table',
                'verbose_name_plural': 'sites routing table',
            },
            managers=[
                ('objects', hacs.models.SiteRoutingTableManager()),
            ],
        ),
        migrations.AddField(
            model_name='contenttyperoutingtable',
            name='route',
            field=models.ForeignKey(db_column='route_id', db_constraint=False, on_delete=django.db.models.deletion.CASCADE, related_name='hacs_route_contenttypes', to='hacs.RoutingTable'),
        ),
        migrations.AddField(
            model_name='contenttyperoutingtable',
            name='site',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='hacs_site_contenttypes_at_routing_table', to='sites.Site'),
        ),
        migrations.AlterUniqueTogether(
            name='contenttyperoutingtable',
            unique_together=set([('site', 'content_type', 'object_id')]),
        ),
    ]



# -*- coding: utf-8 -*-
# ++ This file `utils.py` is generated at 3/5/16 10:02 AM ++
import os
import re
import copy
import glob
import json
import datetime
import warnings
import importlib
import collections
from django.apps import apps
from django.utils import six
from django.conf import settings
from django.utils import timezone
from django.utils._os import safe_join
from django.utils.encoding import smart_bytes
from django.contrib.auth import get_user_model
from django.core.serializers.json import DjangoJSONEncoder
from django.utils.module_loading import module_has_submodule

from .models import SiteRoutingTable
from .globals import HACS_SITE_CACHE
from .defaults import HACS_FALLBACK_URLCONF
from .defaults import HACS_GENERATED_URLCONF_DIR
from .globals import HACS_GENERATED_FILENAME_PREFIX

__author__ = "Md Nazrul Islam<connect2nazrul@gmail.com>"

UserModel = get_user_model()

urlconf_template = """# -*- coding: utf-8 -*-
from django.conf.urls import include
from django.conf.urls import url
from django.contrib import admin

__author__ = "Md Nazrul Islam<connect2nazrul@gmail.com>"

admin.autodiscover()
urlpatterns = [
{urlpatterns}
]

{handlers}
"""


def set_site_urlconf(site):
    """
    :param site:
    :return:
    """
    try:
        site_route = SiteRoutingTable.objects.get(site=site)
    except SiteRoutingTable.DoesNotExist:
        warnings.warn(
                '%s has not assigned any route yet!' % site.domain
        )
        try:
            HACS_SITE_CACHE[site.domain].update(
                {'urlconf': getattr(settings, 'HACS_FALLBACK_URLCONF', HACS_FALLBACK_URLCONF)}
            )
        except KeyError:
            HACS_SITE_CACHE[site.domain] = \
                {'urlconf': getattr(settings, 'HACS_FALLBACK_URLCONF', HACS_FALLBACK_URLCONF)}
    else:
        # we are checking if file need to be created
        generate_urlconf_file_on_demand(site_route.route)
        generated_urlconf_module = get_generated_urlconf_module(get_generated_urlconf_file(site_route.route.route_name))

        try:
            HACS_SITE_CACHE[site.domain].update({'urlconf': generated_urlconf_module})
        except KeyError:
            HACS_SITE_CACHE[site.domain] = {'urlconf': generated_urlconf_module}


def get_site_urlconf(site):
    """
    @Linked to `lru_wrapped`
    :param site
    """
    try:
        # serve from cache
        return HACS_SITE_CACHE[site.domain]['urlconf']

    except KeyError:
        set_site_urlconf(site)
        return HACS_SITE_CACHE[site.domain]['urlconf']


def get_group_key(request, group, prefix='hacl', suffix=None):
    """
    @Linked to `lru_wrapped`
    :param request:
    :param group:
    :param prefix:
    :param suffix:
    :return:
    """
    if suffix is None:
        suffix = ''
    return "{prefix}:site_{site_id}:group_{group_id}{suffix}".\
        format(prefix=prefix, site_id=request.site.id, group_id=group.id, suffix=suffix)


def get_user_key(request, prefix='hacl', suffix=None):
    """
    @Linked to `lru_wrapped`
    :param request:
    :param prefix:
    :param suffix:
    :return:
    """
    if suffix is None:
        suffix = ''
    return "{prefix}:site_{site_id}:user_{user_id}{suffix}".\
        format(prefix=prefix, site_id=request.site.id, user_id=request.user.is_authenticated() and request.user.id or 0,
               suffix=suffix)


def get_generated_urlconf_file(route_name, prefix=None):
    """
    @Linked to `lru_wrapped`
    :param route_name
    :param prefix
    :return:
    """
    route_name = sanitize_filename(route_name)
    if prefix:
        prefix = sanitize_filename(prefix)
    else:
        prefix = HACS_GENERATED_FILENAME_PREFIX

    return safe_join(getattr(settings, 'HACS_GENERATED_URLCONF_DIR', HACS_GENERATED_URLCONF_DIR), "%s_%s_urls.py" %
                     (prefix, route_name))


def get_generated_urlconf_module(filename, validation=True):
    """
    @Linked to `lru_wrapped`
    :param filename:
    :param validation:
    :return:
    """
    if validation and not os.path.exists(filename):
        raise OSError("Specified file: %s does't exists!" % filename)

    return filename.split('/')[-1].split('.')[0]


def generate_urlconf_file_on_demand(route):
    """
    """
    generated_urlconf_file = get_generated_urlconf_file(route.route_name)
    route_updated_on = route.updated_on

    if route_updated_on:
        # Making Native datetime from offset-aware datetime.
        # See http://goo.gl/eHOIe2
        route_updated_on = route_updated_on.astimezone(timezone.utc).replace(tzinfo=None)

    # we are checking if file need to be created
    if not os.path.exists(generated_urlconf_file) or \
        (os.path.exists(generated_urlconf_file) and route_updated_on and
                 datetime.datetime.utcfromtimestamp(
                     os.path.getmtime(generated_urlconf_file)) < route_updated_on):
        # OK: we need to generate the file
        generate_urlconf_file(generated_urlconf_file, route)


def generate_urlconf_file(filename, route):
    """
    :param filename:
    :param route:
    :return:
    """
    with open(filename, 'w') as f:

        urlpatterns = ""

        for url in route.urls:
            url_module = url['url_module']
            if url.get('app_name'):
                if isinstance(url_module, six.string_types):
                    url_module = (url_module, url.get('app_name'))
                elif isinstance(url_module, (tuple, list, set)) and len(url_module) == 1:
                    url_module = tuple(url_module) + (url.get('app_name'))

            if not isinstance(url_module, six.string_types):
                if isinstance(url_module, (list, set)):
                    url_module = tuple(url_module)
                if isinstance(url_module, tuple):
                    if len(url_module) > 2:
                        url_module = url_module[0:2]
                        if not url.get('namespace'):
                            url['namespace'] = url_module[2]

                url_module = smart_bytes(url_module)
            else:
                # `admin.site.urls` is kind of special case, i. admin autodiscover
                if url_module != 'admin.site.urls':
                    url_module = "'%s'" % url_module

            urlpatterns += "\turl(r'{prefix}', include({url_module}{namespace})),\n".\
                format(prefix=url.get('prefix') + '/' if url.get('prefix', None) else '^', url_module=url_module,
                       namespace=url.get('namespace') and ", namespace='%s'" % url.get('namespace', None) or '')

        error_handlers = ''
        if route.handlers:
            for name, handler in route.handlers.items() or {}.items():
                error_handlers += "%s=\"%s\"\n" % (name, handler)

        f.write(copy.copy(urlconf_template).format(urlpatterns=urlpatterns, handlers=error_handlers))


def sanitize_filename(filename):
    """
    :param filename:
    :return:
    """
    filename = re.sub(r'[^A-Za-z0-9\-_]+', '', filename)

    return filename.replace('-', '_')


def get_installed_apps_urlconf(pattern=r'*urls.py', to_json=False, exclude=()):
    """
    @Linked to `lru_wrapped`
    :param pattern:
    :param to_json:
    :param exclude: apps those should be excluded
    :return:
    """
    result = collections.deque()
    handlers = (
        "handler400",
        "handler403",
        "handler404",
        "handler500",
    )
    if exclude and isinstance(exclude, six.string_types):
        exclude = (exclude, )

    for app in apps.get_app_configs():

        if app.label in exclude:
            continue

        # First try to files
        urlconf_modules = map(lambda x: '.'.join((app.name, x.replace(app.path, '').lstrip('/').split('.')[0]),),
                              glob.glob(os.path.join(app.path, pattern)))

        urlconf_dir = os.path.join(app.path, 'urls')

        if os.path.exists(urlconf_dir) and module_has_submodule(app.module, 'urls'):
            # we will try from urls module
            urls_module = importlib.import_module('.'.join((app.name, 'urls', )))
            if hasattr(urls_module, 'urlpatterns'):
                urlconf_modules += ['.'.join((app.name, 'urls', )), ]

            urlconf_modules += map(
                lambda x: '.'.join((app.name, 'urls', x.replace(app.path, '').lstrip('/').split('.')[0]), ),
                glob.glob(os.path.join(urlconf_dir, pattern)))

        if not urlconf_modules:
            continue

        for urlconf in urlconf_modules:

            urlconf_module = importlib.import_module(urlconf)
            if not hasattr(urlconf_module, 'urlpatterns'):
                continue

            error_handlers = collections.defaultdict(dict)
            for handler in handlers:
                if hasattr(urlconf_module, handler):
                    error_handlers[handler] = getattr(urlconf_module, handler)

            item = collections.namedtuple('app_' + app.name.replace('.', '_'), ['module', 'app_label', 'prefix', 'error_handlers'])
            result.append(item(module=urlconf, app_label=app.label, error_handlers=error_handlers, prefix=None))

    if to_json:
        return json.dumps(list(result), cls=DjangoJSONEncoder)
    else:
        return tuple(result)


def get_user_object(username_or_email_or_mobile, silent=True):
    """
    :param
    :param username_or_email_or_mobile:
    :param silent:
    :return:
    """
    try:
        return UserModel.objects.get(**{UserModel.USERNAME_FIELD: username_or_email_or_mobile})
    except UserModel.DoesNotExist:
        if not silent:
            raise
        return None

__all__ = (
    "set_site_urlconf",
    "get_site_urlconf",
    "get_group_key",
    "get_user_key",
    "get_generated_urlconf_file",
    "get_generated_urlconf_module",
    "generate_urlconf_file",
    "generate_urlconf_file_on_demand",
    "sanitize_filename",
    "get_installed_apps_urlconf",
    "get_user_object",
)




# -*- coding: utf-8 -*-
# ++ This file `export_routes.py` is generated at 4/18/16 4:20 PM ++
import os
import time
import json
import shutil
from django.utils import six
from django.apps import apps
from django.db.models import Q
from django.conf import settings
from django.core import serializers
from django.contrib.sites.models import Site
from django.contrib.auth.models import Group
from django.utils.encoding import smart_text
from django.contrib.auth import get_user_model
from django.core.management import BaseCommand
from django.core.management import CommandError
from django.utils.module_loading import import_string
from django.contrib.contenttypes.models import ContentType
from django.core.serializers.json import DjangoJSONEncoder

from hacs.models import RoutingTable
from hacs.models import SiteRoutingTable
from hacs.models import ContentTypeRoutingTable
from hacs.defaults import HACS_SERIALIZED_ROUTING_DIR
from hacs.defaults import HACS_USER_OBJECT_QUERY_CALLABLE

__author__ = "Md Nazrul Islam<connect2nazrul@gmail.com>"

UserModel = get_user_model()


class Command(BaseCommand):
    """ """
    help = "HACS: Command tool for exporting"
    can_import_settings = True

    def add_arguments(self, parser):
        """"""
        # Add option for source file
        parser.add_argument(
            '-d',
            '--destination',
            action='store',
            dest='destination',
            help='The destination where serialized data will be stored physically. Value could be directory or absolute'
                 ' file path'
        )
        # Add exclude app option
        parser.add_argument(
            '-a',
            '--exclude-apps',
            action='store',
            dest='exclude_apps',
            nargs='+',
            help='List of apps you want to ignore. Example: --exclude-apps app1 app2 ..'
        )
        # Add exclude site options
        parser.add_argument(
            '-s',
            '--exclude-sites',
            action='store',
            dest='exclude_sites',
            nargs='+',
            help='List of sites you want to ignore. Value must be `domain name` of site not ID '
                 'Example: --exclude-sites site1 site2 ..'
        )
        # Add exclude user group options
        parser.add_argument(
            '-g',
            '--exclude-groups',
            action='store',
            dest='exclude_groups',
            nargs='+',
            help='List of user groups you want to ignore. Value must group name not ID.'
                 ' Example: --exclude-groups group1 group2 ..'
        )
        # Add exclude user options
        parser.add_argument(
            '-u',
            '--exclude-users',
            action='store',
            dest='exclude_users',
            nargs='+',
            help='List of users you want to ignore. Value must username or email not ID.'
                 ' Example: --exclude-users user1 user2 ..'
        )
        # Add exclude route options
        parser.add_argument(
            '-r',
            '--exclude-routes',
            action='store',
            dest='exclude_routes',
            nargs='+',
            help='List of routes you want to ignore. Value must provide route name not ID.'
                 ' Example: --exclude-routes route1 route2 ..'
        )
        # Add option for print only instead of storing
        parser.add_argument(
            '-p',
            '--print-only',
            action='store_true',
            dest="print_only",
            help="Use this option if you want to just print serialized data instead of store to filesystem"
        )
        # Add option for extended natural key
        # By default serialization will be here with enabling `natural primary and foreign key` but we may need more.
        # For example: Site model has no natural key, in ContentType `GenericForeignKey` relation's object_id does't
        # support natural key.
        parser.add_argument(
            '-x',
            '--extended-natural-keys',
            action='store_true',
            dest='extended_natural_keys',
            help='Enable extended natural keys.I.e for Site, objects_is'
        )
        # Add option for output format
        parser.add_argument(
            '-f',
            '--format',
            action='store',
            dest='output_format',
            choices=('json', 'xml',),
            default='json',
            help='Output format of serialized data or type of serializer. by default is json'
        )
        # Add option for no natural  primary keys
        parser.add_argument(
            '--no-natural-primary',
            action='store_true',
            dest='no_natural_primary_keys',
            help='By default this tool uses natural primary keys, but you can omit by using this option'
        )
        # Add option for no natural foreign keys
        parser.add_argument(
            '--no-natural-foreign',
            action='store_true',
            dest='no_natural_foreign_keys',
            help='By default this tool uses natural foreign keys, but you can omit by using this option'
        )

    def handle(self, *args, **options):
        """"""
        self._validate(*args, **options)
        objects = self.collect_all_objects_list(*args, **options)

        if not objects:
            self.stdout.write('Nothing to export. Empty result.')
            return
        filename, serializer = self._calculate_output_filename(*args, **options)
        serializer_options = {}

        if not options['no_natural_primary_keys']:
            serializer_options['use_natural_primary_keys'] = True

        if not options['no_natural_foreign_keys']:
            serializer_options['use_natural_foreign_keys'] = True

        if filename:
            with open(filename, 'w') as f:
                serializers.serialize(serializer, objects, stream=f, **serializer_options)
            if options['extended_natural_keys'] and serializer == 'json':
                with open(filename, 'r') as f:
                    objects = json.load(f, encoding=settings.DEFAULT_CHARSET)
                    self._wrap_extended_natural_keys(objects)

                    with open(filename + '.tmp', 'w') as fp:
                        json.dump(objects, fp=fp, cls=DjangoJSONEncoder)

                shutil.move(filename + '.tmp', filename)

        else:
            data = serializers.serialize(serializer, objects, **serializer_options)
            if options['extended_natural_keys'] and serializer == 'json':
                data = json.loads(data)
                self._wrap_extended_natural_keys(data)
                data = json.dumps(data, cls=DjangoJSONEncoder)
            self.stdout.write(data)

    def collect_all_objects_list(self, *args, **kwargs):
        """"""
        filters = {}
        exclude_routes = kwargs['exclude_routes']
        if exclude_routes is not None and isinstance(exclude_routes, six.string_types):
            exclude_routes = (exclude_routes,)
        if exclude_routes:
            filters['route_name__in'] = exclude_routes

        routing_table_queryset = RoutingTable.objects.exclude(**filters)

        if filters.get('route_name__in', None):
            filters['route__in'] = [RoutingTable.objects.get_by_natural_key(x) for x in filters.get('route_name__in')]
            del filters['route_name__in']

        exclude_sites = kwargs['exclude_sites']
        if exclude_sites is not None and isinstance(exclude_sites, six.string_types):
            exclude_sites = (exclude_sites,)

        if exclude_sites:
            filters['site__in'] = [Site.objects.get(domain=x) for x in exclude_sites]

        site_routing_queryset = SiteRoutingTable.objects.exclude(**filters)

        filter_arg = ()
        exclude_groups = kwargs['exclude_groups']
        if exclude_groups is not None and isinstance(exclude_groups, six.string_types):
            exclude_groups = (exclude_groups,)

        if exclude_groups:
            exclude_groups = [Group.objects.get_by_natural_key(x).pk for x in exclude_groups]
            filter_arg = (Q(content_type=ContentType.objects.get_for_model(Group), object_id__in=exclude_groups), )

        exclude_users = kwargs['exclude_users']
        if exclude_users is not None and isinstance(exclude_users, six.string_types):
            exclude_users = (exclude_users,)
        if exclude_users:
            exclude_users = [import_string(getattr(settings, 'HACS_USER_OBJECT_QUERY_CALLABLE',
                                                   HACS_USER_OBJECT_QUERY_CALLABLE))(x, silent=False).pk
                             for x in exclude_users]
            if filter_arg:
                filter_arg = (filter_arg[0] | Q(content_type=ContentType.objects.get_for_model(UserModel),
                                            object_id__in=exclude_users), )
            else:
                filter_arg = (Q(content_type=ContentType.objects.get_for_model(UserModel),
                                object_id__in=exclude_users), )

        contenttype_routing_queryset = ContentTypeRoutingTable.objects.exclude(*filter_arg, **filters)

        return list(routing_table_queryset) + list(site_routing_queryset) + list(contenttype_routing_queryset)

    def _validate(self, *args, **kwargs):
        """
        :param args:
        :param kwargs:
        :return:
        """
        if not kwargs['destination'] and not kwargs['print_only']:
            raise CommandError("You have to pass either destination of output file or enable print only")

        if kwargs['destination']:
            if '/' not in kwargs['destination'] and os.path.splitext(kwargs['destination'])[1]:
                # We are sure only file name is provided
                if not getattr(settings, 'HACS_SERIALIZED_ROUTING_DIR', HACS_SERIALIZED_ROUTING_DIR):
                    raise CommandError("You provide destination file name, not with full path, in that case you must "
                                       "define `HACS_SERIALIZED_ROUTING_DIR` at django settings")

        exclude_apps = kwargs['exclude_apps']
        if exclude_apps is not None and isinstance(exclude_apps, six.string_types):
            exclude_apps = (exclude_apps,)
        if exclude_apps:
            for app_label in exclude_apps:
                try:
                    apps.get_app_config(app_label)
                except LookupError as err:
                    raise CommandError("Invalid app: " + smart_text(err))

        exclude_sites = kwargs['exclude_sites']
        if exclude_sites is not None and isinstance(exclude_sites, six.string_types):
            exclude_sites = (exclude_sites,)

        if exclude_sites:
            for site in exclude_sites:
                try:
                    Site.objects.get(domain=site)
                except Site.DoesNotExist:
                    raise CommandError("Specified site `%s` doesn't exists!" % site)

        exclude_groups = kwargs['exclude_groups']
        if exclude_groups is not None and isinstance(exclude_groups, six.string_types):
            exclude_groups = (exclude_groups,)

        if exclude_groups:
            for group in exclude_groups:
                try:
                    Group.objects.get_by_natural_key(group)
                except Group.DoesNotExist:
                    raise CommandError("Specified group `%s` doesn't exists!" % group)

        exclude_users = kwargs['exclude_users']
        if exclude_users is not None and isinstance(exclude_users, six.string_types):
            exclude_users = (exclude_users,)
        if exclude_users:
            for user in exclude_users:
                try:
                    import_string(getattr(settings, 'HACS_USER_OBJECT_QUERY_CALLABLE',
                                          HACS_USER_OBJECT_QUERY_CALLABLE))(user, silent=False)
                except UserModel.DoesNotExist:
                    raise CommandError("Specified user `%s` doesn't exists!" % user)

        exclude_routes = kwargs['exclude_routes']
        if exclude_routes is not None and isinstance(exclude_routes, six.string_types):
            exclude_routes = (exclude_routes,)
        if exclude_routes:
            for route in exclude_routes:
                try:
                    RoutingTable.objects.get_by_natural_key(route)
                except RoutingTable.DoesNotExist:
                    raise CommandError("Specified route `%s` doesn't exists!" % route)

        if kwargs['extended_natural_keys'] and (kwargs['no_natural_primary_keys'] or kwargs['no_natural_foreign_keys']):
            raise CommandError("--extended-natural-keys could be only applied, if enable natural PK and FK")

    def _calculate_output_filename(self, *args, **kwargs):
        """ """

        if kwargs['print_only']:
            # we don't care whether has destination
            return None, kwargs['output_format']

        filename = kwargs['destination']
        if filename:
            if '/' not in filename and os.path.splitext(filename)[1]:
                # We are sure only file name is provided
                filename = os.path.join(getattr(settings, 'HACS_SERIALIZED_ROUTING_DIR', HACS_SERIALIZED_ROUTING_DIR),
                                        filename)
            elif '/' in filename and not os.path.splitext(filename)[1]:
                filename = os.path.join(filename,
                                        'hacs_serialized_routes_generated_%s.%s' %
                                        (time.time(), kwargs['output_format']))

        return filename, kwargs['output_format']

    def _wrap_extended_natural_keys(self, entries):

        """"""
        _allowed_models = (
            "%s.%s" % (SiteRoutingTable._meta.app_label, SiteRoutingTable._meta.model_name),
            "%s.%s" % (ContentTypeRoutingTable._meta.app_label, ContentTypeRoutingTable._meta.model_name),
        )

        for entry in entries:

            if entry['model'].lower() not in _allowed_models:
                continue
            entry['fields']['site'] = (Site.objects.get(pk=entry['fields']['site']).domain, )

            if entry['model'].lower() == _allowed_models[1]:
                # Check if for Auth User Model
                if UserModel._meta.app_label in entry['fields']['content_type'] and \
                        UserModel._meta.model_name in entry['fields']['content_type']:
                    entry['fields']['object_id'] = \
                    (getattr(UserModel.objects.get(pk=entry['fields']['object_id']), UserModel.USERNAME_FIELD), )

                elif Group._meta.app_label in entry['fields']['content_type'] and Group._meta.model_name in \
                    entry['fields']['content_type']:
                    entry['fields']['object_id'] = Group.objects.get(pk=entry['fields']['object_id']).natural_key()



# -*- coding: utf-8 -*-
"""Installer for the django-hacs package."""
import os
from setuptools import setup


def get_version():
    """"""
    return __import__('hacs').__version__

long_description = (
    open('README.rst').read() +
    '\n' +
    'Contributors\n' +
    '============\n' +
    '\n' +
    open('CONTRIBUTORS.rst').read() +
    '\n' +
    open('CHANGES.rst').read() +
    '\n')


def get_packages(package):
    """
    Return root package and all sub-packages.
    """
    return [dirpath
            for dirpath, dirnames, filenames in os.walk(package)
            if os.path.exists(os.path.join(dirpath, '__init__.py'))]


def get_package_data(package):
    """
    Return all files under the root package, that are not in a
    package themselves.
    """
    walk = [(dirpath.replace(package + os.sep, '', 1), filenames)
            for dirpath, dirnames, filenames in os.walk(package)
            if not os.path.exists(os.path.join(dirpath, '__init__.py'))]

    filepaths = []
    for base, filenames in walk:
        filepaths.extend([os.path.join(base, filename)
                          for filename in filenames])
    return {package: filepaths}




setup(
    name='django-hacs',
    version=get_version(),
    description="Hybrid Access Control System for Django",
    long_description=long_description,
    zip_safe=False,
    classifiers=[
        'Development Status :: 3 - Alpha',
        'Environment :: Web Environment',
        'Framework :: Django',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: GNU Lesser General Public License v3 (LGPLv3)',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Topic :: Internet :: WWW/HTTP',
        'Topic :: Internet :: WWW/HTTP :: Dynamic Content',
        'Topic :: Internet :: WWW/HTTP :: WSGI',
        'Topic :: Software Development :: Libraries :: Application Frameworks',
        'Topic :: Software Development :: Libraries :: Python Modules',
    ],
    keywords='Python Django ACL ACS IAM Security Authentication Authorization',
    author='Md Nazrul Islam',
    author_email='email2nazrul@gmail.com',
    url='https://pypi.python.org/pypi/django-hacs',
    license='GPL version 3',
    packages=get_packages('hacs'),
    package_data=get_package_data('hacs'),
    install_requires=[
        'django',
    ],
    extras_require={
        'test': ['pytest', 'pytest-django'],
    }
)



# -*- coding: utf-8 -*-
# ++ This file `urls.py` is generated at 3/18/16 3:49 PM ++
from django.conf.urls import url

from .views import IndexView
from .views import AboutView
__author__ = "Md Nazrul Islam<connect2nazrul@gmail.com>"

urlpatterns = [
    url(r'^', name='index', view=IndexView.as_view()),
    url(r'^about/', name='about', view=AboutView.as_view())
]

handler403 = "hacs.views.errors.permission_denied"
handler404 = "hacs.views.errors.page_not_found"



# -*- coding: utf-8 -*-
from django.views.defaults import page_not_found
from django.views.defaults import server_error
from django.views.defaults import bad_request
from django.views.defaults import permission_denied

__author__ = "Md Nazrul Islam<connect2nazrul@gmail.com>"

ERROR_404_TEMPLATE_NAME = '404.html'
ERROR_403_TEMPLATE_NAME = '403.html'
ERROR_400_TEMPLATE_NAME = '400.html'
ERROR_500_TEMPLATE_NAME = '500.html'



# -*- coding: utf-8 -*-
# ++ This file `import_routes.py` is generated at 4/9/16 6:11 AM ++
import os
import glob
import json
import tempfile
from django.utils import six
from django.apps import apps
from django.conf import settings
from django.core import serializers
from django.contrib.auth.models import Group
from django.utils.encoding import smart_text
from django.contrib.sites.models import Site
from django.contrib.auth import get_user_model
from django.utils.module_loading import import_string
from django.contrib.contenttypes.models import ContentType
from django.core.management.base import BaseCommand, CommandError

from hacs.models import RoutingTable
from hacs.models import SiteRoutingTable
from hacs.models import ContentTypeRoutingTable
from hacs.defaults import HACS_SERIALIZED_ROUTING_DIR
from hacs.defaults import HACS_USER_OBJECT_QUERY_CALLABLE
from hacs.globals import HACS_SERIALIZED_ROUTE_DIR_NAME

__author__ = "Md Nazrul Islam<connect2nazrul@gmail.com>"

UserModel = get_user_model()


class Command(BaseCommand):
    """"""
    help = "HACS: Command tool for importing"
    can_import_settings = True

    def add_arguments(self, parser):
        """
        :param parser:
        :return:
        """
        # Add option for source file
        parser.add_argument(
            '-S',
            '--source',
            action='store',
            dest='source',
            help='Importable source file location, could be absolute path or app prefix path(prefix '
                 'should be separated by :). If source is provided all other arguments except verbose will be ignored'
        )
        # Add exclude app option
        parser.add_argument(
            '-a',
            '--exclude-apps',
            action='store',
            dest='exclude_apps',
            nargs='+',
            help='List of apps you want to ignore. Example: --exclude-apps app1 app2 ..'
        )
        # Add exclude site options
        parser.add_argument(
            '-s',
            '--exclude-sites',
            action='store',
            dest='exclude_sites',
            nargs='+',
            help='List of sites you want to ignore. Value must be `domain name` of site not ID '
                 'Example: --exclude-sites site1 site2 ..'
        )
        # Add exclude user group options
        parser.add_argument(
            '-g',
            '--exclude-groups',
            action='store',
            dest='exclude_groups',
            nargs='+',
            help='List of user groups you want to ignore. Value must group name not ID.'
                 ' Example: --exclude-groups group1 group2 ..'
        )
        # Add exclude user options
        parser.add_argument(
            '-u',
            '--exclude-users',
            action='store',
            dest='exclude_users',
            nargs='+',
            help='List of users you want to ignore. Value must username or email not ID.'
                 ' Example: --exclude-users user1 user2 ..'
        )
        # Add walking to app directory discover option
        parser.add_argument(
            '--omit-app-dir-walking',
            action='store_true',
            default=False,
            dest="omit_app_dir_walking",
            help="By this tool walking over all installed apps to discover importable file, "
                 "so you omit this by using this option"
        )

    def handle(self, *args, **options):
        """
        :param args:
        :param options:
        :return:
        """
        self._validate(*args, **options)
        importable_files = self._get_importable_files(*args, **options)

        for serialized_file in importable_files:
            # we will normalized natural keys to PK those are not supported by django by default
            # For example User, Site and object id of contenttype relationship.
            # Right now normalization is applied for json file only
            if os.path.splitext(serialized_file)[1][1:].lower() == 'json':
                _temp_file = tempfile.mktemp(suffix='.json')
                with open(serialized_file, 'r') as f:
                    objects = json.load(f)
                    self._normalize_unwanted_natural_keys(objects)
                    with open(_temp_file, 'w') as t_f:
                        json.dump(objects, t_f)
                file_like_obj = open(_temp_file, 'r')
                os.unlink(_temp_file)
            else:
                file_like_obj = open(serialized_file, 'r')

            with file_like_obj:
                deserialized_objects = serializers.deserialize(os.path.splitext(serialized_file)[1][1:], file_like_obj)
                for deserialized_obj in deserialized_objects:
                    if not self._object_validate(deserialized_obj.object, *args, **options):
                        continue
                    deserialized_obj.save()

    def _object_validate(self, obj, *args, **kwargs):
        """
        :param obj
        :param args:
        :param kwargs:
        :return:
        """
        if not isinstance(obj, (RoutingTable, SiteRoutingTable, ContentTypeRoutingTable, )):
            return False

        exclude_apps = kwargs['exclude_apps']
        if exclude_apps is not None and isinstance(exclude_apps, six.string_types):
            exclude_apps = (exclude_apps, )
            # We are not doing with excluding apps right now.
            pass

        exclude_sites = kwargs['exclude_sites']
        if exclude_sites is not None and isinstance(exclude_sites, six.string_types):
            exclude_sites = [exclude_sites, ]

        if exclude_sites and not isinstance(obj, RoutingTable):
            if obj.site.domain in exclude_sites:
                return False

        exclude_groups = kwargs['exclude_groups']
        if exclude_groups is not None and isinstance(exclude_groups, six.string_types):
            exclude_groups = (exclude_groups, )

        if exclude_groups and isinstance(obj, ContentTypeRoutingTable):
            content_type = ContentType.objects.get_for_model(Group)
            if obj.content_type == content_type and Group.objects.get(pk=obj.object_id).name in exclude_groups:
                return False

        exclude_users = kwargs['exclude_users']
        if exclude_users is not None and isinstance(exclude_users, six.string_types):
            exclude_users = (exclude_users, )

        if exclude_users and isinstance(obj, ContentTypeRoutingTable):

            content_type = ContentType.objects.get_for_model(UserModel)
            if obj.content_type == content_type and \
                    getattr(UserModel.objects.get(pk=obj.object_id), UserModel.USERNAME_FIELD) in exclude_users:
                return False

        return True

    def _validate(self, *args, **kwargs):
        """
        :return:
        """
        if not kwargs['source'] and not getattr(settings, 'HACS_SERIALIZED_ROUTING_DIR',HACS_SERIALIZED_ROUTING_DIR) \
            and kwargs['omit_app_dir_walking']:

            raise CommandError("Required value is missing! you have to either provide source file or define "
                               "routing directory name or enable app discovering")

        exclude_apps = kwargs['exclude_apps']
        if exclude_apps is not None and isinstance(exclude_apps, six.string_types):
            exclude_apps = (exclude_apps, )
        if exclude_apps:
            for app_label in exclude_apps:
                try:
                    apps.get_app_config(app_label)
                except LookupError as err:
                    raise CommandError(smart_text("Invalid app: ") + smart_text(err))

        exclude_sites = kwargs['exclude_sites']
        if exclude_sites is not None and isinstance(exclude_sites, six.string_types):
            exclude_sites = (exclude_sites, )

        if exclude_sites:
            for site in exclude_sites:
                try:
                    Site.objects.get(domain=site)
                except Site.DoesNotExist:
                    raise CommandError("Specified site `%s` doesn't exists!" % site)

        exclude_groups = kwargs['exclude_groups']
        if exclude_groups is not None and isinstance(exclude_groups, six.string_types):
            exclude_groups = (exclude_groups,)

        if exclude_groups:
            for group in exclude_groups:
                try:
                    Group.objects.get_by_natural_key(group)
                except Group.DoesNotExist:
                    raise CommandError("Specified group `%s` doesn't exists!" % group)

        exclude_users = kwargs['exclude_users']
        if exclude_users is not None and isinstance(exclude_users, six.string_types):
            exclude_users = (exclude_users,)
        if exclude_users:
            for user in exclude_users:
                try:
                    import_string(getattr(settings, 'HACS_USER_OBJECT_QUERY_CALLABLE',
                                          HACS_USER_OBJECT_QUERY_CALLABLE))(user, silent=False)
                except UserModel.DoesNotExist:
                    raise CommandError("Specified user `%s` doesn't exists!" % user)

    def _get_importable_files(self, *args, **kwargs):
        """
        :param args:
        :param kwargs:
        :return:
        """
        importable_files = []
        if kwargs['source']:

            _temp = kwargs['source']
            if os.path.exists(_temp):
                importable_files.append(_temp)

            elif not os.path.exists(_temp) and \
                getattr(settings, 'HACS_SERIALIZED_ROUTING_DIR', HACS_SERIALIZED_ROUTING_DIR):
                if os.path.exists(os.path.join(getattr(settings, 'HACS_SERIALIZED_ROUTING_DIR',
                                                       HACS_SERIALIZED_ROUTING_DIR), _temp)):
                    importable_files.append(os.path.join(getattr(settings, 'HACS_SERIALIZED_ROUTING_DIR',
                                                 HACS_SERIALIZED_ROUTING_DIR), _temp))
            else:

                if 2 == len(_temp.split(':')):

                    try:
                        app = apps.get_app_config(_temp.split(':')[0])
                    except LookupError as err:
                        raise CommandError('Invalid app path pattern: ' + smart_text(err) + '. Provided path is %s' % _temp)
                    else:
                        if os.path.exists(os.path.join(app.path, _temp.split(':')[1])):
                            importable_files.append(os.path.join(app.path, _temp.split(':')[1]))

            if not importable_files:
                raise CommandError('Invalid source path specified!. You can supply as absolute path or app bounded path '
                                   '(app-label:relative path to app)')
            # We ignore further file traversing
        else:

            if getattr(settings, 'HACS_SERIALIZED_ROUTING_DIR', HACS_SERIALIZED_ROUTING_DIR):

                importable_files = glob.glob(os.path.join(
                    getattr(settings, 'HACS_SERIALIZED_ROUTING_DIR', HACS_SERIALIZED_ROUTING_DIR),
                    r'*.json')) + glob.glob(os.path.join(
                    getattr(settings, 'HACS_SERIALIZED_ROUTING_DIR', HACS_SERIALIZED_ROUTING_DIR),
                    r'*.xml'))
            if not kwargs['omit_app_dir_walking']:

                for app in apps.get_app_configs():

                    if os.path.exists(os.path.join(app.path, HACS_SERIALIZED_ROUTE_DIR_NAME)):

                        importable_files += glob.glob(os.path.join(app.path, HACS_SERIALIZED_ROUTE_DIR_NAME,
                                                                   r'*.json'))
                        importable_files += glob.glob(os.path.join(app.path, HACS_SERIALIZED_ROUTE_DIR_NAME,
                                                                   r'*.xml'))

        return importable_files

    def _normalize_unwanted_natural_keys(self, entries):
        """ """
        _allowed_models = (
            "%s.%s" % (SiteRoutingTable._meta.app_label, SiteRoutingTable._meta.model_name),
            "%s.%s" % (ContentTypeRoutingTable._meta.app_label, ContentTypeRoutingTable._meta.model_name),
        )
        for entry in entries:
            if entry['model'].lower() in _allowed_models:

                if isinstance(entry['fields']['site'], (list, tuple, )):
                    entry['fields']['site'] = Site.objects.get(domain=entry['fields']['site'][0]).pk

                if entry['model'].lower() == _allowed_models[1]:
                    if isinstance(entry['fields']['object_id'], (list, tuple)):
                        # Check if for Auth User Model
                        if UserModel._meta.app_label in entry['fields']['content_type'] and \
                                UserModel._meta.model_name in entry['fields']['content_type']:
                            entry['fields']['object_id'] = import_string(getattr(settings,
                                                                               'HACS_USER_OBJECT_QUERY_CALLABLE',
                                                                               HACS_USER_OBJECT_QUERY_CALLABLE)
                                                                       )(*entry['fields']['object_id']).pk

                        elif Group._meta.app_label in entry['fields']['content_type'] and Group._meta.model_name in \
                            entry['fields']['content_type']:
                            entry['fields']['object_id'] = \
                                Group.objects.get_by_natural_key(*entry['fields']['object_id']).pk







# -*- coding: utf-8 -*-
# ++ This file `middleware.py` is generated at 3/3/16 6:05 AM ++
import logging
import warnings
from django.apps import apps
from django.utils import six
from django.conf import settings
from django.core.cache import caches
from django.contrib.auth.models import Group
from django.contrib.auth import get_user_model
try:
    # expect django version 1.10.x or higher
    from django.urls import get_resolver, Resolver404
except ImportError:
    from django.core.urlresolvers import get_resolver, Resolver404
from django.utils.functional import cached_property
from django.core.exceptions import ImproperlyConfigured
from django.utils.translation import ugettext_lazy as _
from django.contrib.contenttypes.models import ContentType
from django.contrib.sites.shortcuts import get_current_site
from .globals import HACS_APP_NAME
from .models import ContentTypeRoutingTable
from .defaults import HACS_FALLBACK_URLCONF
from .defaults import HACS_CACHE_SETTING_NAME
from .lru_wrapped import get_user_key
from .lru_wrapped import get_group_key
from .utils import generate_urlconf_file_on_demand
from .lru_wrapped import get_generated_urlconf_file
from .lru_wrapped import get_generated_urlconf_module
from .lru_wrapped import get_site_urlconf

__author__ = "Md Nazrul Islam<connect2nazrul@gmail.com>"

logger = logging.getLogger(u'hacs.middleware')
UserModel = get_user_model()


class DynamicRouteMiddleware(object):
    """ """
    name = 'hacs.middleware.DynamicRouteMiddleware'

    def process_request(self, request):
        """
        :param request:
        :return:
        """
        if self._validate:
            pass

        urlconf = get_site_urlconf(getattr(request, 'site', get_current_site(request)))

        request.urlconf = urlconf

    @cached_property
    def _validate(self):
        """
        :return:
        """
        errors = []
        if not apps.is_installed(HACS_APP_NAME):
            errors.append(_("%s need to be added into settings.INSTALLED_APPS" % HACS_APP_NAME))

        if not apps.is_installed('django.contrib.contenttypes'):
            errors.append(_("django.contrib.contenttypes need to be added into settings.INSTALLED_APPS"))

        if not apps.is_installed('django.contrib.sites'):
            errors.append(_("django.contrib.sites need to be added into settings.INSTALLED_APPS"))

        try:
            site_middleware_position = \
                settings.MIDDLEWARE_CLASSES.index('django.contrib.sites.middleware.CurrentSiteMiddleware')
        except ValueError:
            errors.append(_("django.contrib.sites.middleware.CurrentSiteMiddleware need to be "
                            "added into settings.MIDDLEWARE_CLASSES"))
        else:
            if site_middleware_position > settings.MIDDLEWARE_CLASSES.index(self.name):
                errors.append(_('django.contrib.sites.middleware.CurrentSiteMiddleware\'s position should be before ' +
                                self.name))
        if errors:
            raise ImproperlyConfigured(_("Please fix: %s" % ' | '.join(errors)))

        return True


class FirewallMiddleware(object):
    """     """
    name = 'hacs.middleware.FirewallMiddleware'

    def process_request(self, request):
        """
        :param request:
        :return:
        """
        if self._validate:
            pass

        if request.user.is_authenticated():

            request_path = request.path_info
            try:
                user_settings_session = request.session['settings']
            except KeyError:
                request.session['settings'] = dict()
                user_settings_session = request.session['settings']

            try:
                user_urlconf = user_settings_session['urlconf']
            except KeyError:
                # There is no urlconf set yet! now assigning
                self._set_user_urlconf(request)
                user_urlconf = user_settings_session['urlconf']

            user_urlconf = self._calculate_user_urlconf(request_path, user_urlconf)
        else:
            # Anonymous User
            user_urlconf = self._calculate_anonymous_urlconf(request)

        if user_urlconf:
            request.urlconf = user_urlconf
        else:
            # Let's check already set by DynamicRouteMiddleware
            if not getattr(request, 'urlconf', None):
                warnings.warn("urlconf is neither set by DynamicRoutingMiddleware nor from django settings.")
                request.urlconf = getattr(settings, 'HACS_FALLBACK_URLCONF', HACS_FALLBACK_URLCONF)

    def _set_user_urlconf(self, request):
        """
        :param request:
        :return:
        """
        cache = caches[getattr(settings, 'HACS_CACHE_SETTING_NAME', HACS_CACHE_SETTING_NAME)]
        # Try from cache
        user_settings_cache = cache.get(get_user_key(request))
        if user_settings_cache and user_settings_cache.get('urlconf', None):
            request.session['settings']['urlconf'] = user_settings_cache.get('urlconf')
            return None
        # No cache
        if not user_settings_cache:
            user_settings_cache = dict()
        try:
            user_route = ContentTypeRoutingTable.objects.get(
                site=request.site,
                content_type=ContentType.objects.get_for_model(UserModel),
                object_id=request.user.is_authenticated() and request.user.pk or 0)

        except ContentTypeRoutingTable.DoesNotExist:
            # User Has No Route Try from group
            _temp = list()
            for group in request.user.groups.all():
                group_key = get_group_key(request, group)
                # Try from cache
                group_settings_cache = cache.get(group_key, None)
                if group_settings_cache and group_settings_cache.get('urlconf', None):
                    _temp.append(group_settings_cache.get('urlconf'))
                    continue
                # Not found in cache, need to set
                self._set_group_urlconf(request, group)
                if cache.get(group_key, None).get('urlconf', None):
                    _temp.append(cache.get(group_key).get('urlconf'))
            if _temp:
                user_settings_cache['urlconf'] = tuple(_temp)
            else:
                user_settings_cache['urlconf'] = None
        else:
            # Check if urlconf file need to be created
            generate_urlconf_file_on_demand(user_route.route)
            user_settings_cache['urlconf'] = get_generated_urlconf_module(
                get_generated_urlconf_file(user_route.route.route_name))

        # Update Session
        request.session['settings'].update({'urlconf': user_settings_cache['urlconf']})
        # Set Cache
        cache.set(get_user_key(request), user_settings_cache)

    def _set_group_urlconf(self, request, group):
        """
        :param request:
        :param group:
        :return:
        """
        cache = caches[getattr(settings, 'HACS_CACHE_SETTING_NAME', HACS_CACHE_SETTING_NAME)]
        # Try from cache
        cache_key = get_group_key(request, group)
        group_settings_cache = cache.get(cache_key)
        # No cache
        if not group_settings_cache:
            group_settings_cache = dict()
        try:
            group_route = ContentTypeRoutingTable.objects.get(
                content_type=ContentType.objects.get_for_model(Group), object_id=group.id, site=request.site)
        except ContentTypeRoutingTable.DoesNotExist:
            # We do nothing
            pass
        else:
            # we are checking if file need to be created
            generate_urlconf_file_on_demand(group_route.route)
            group_settings_cache['urlconf'] = get_generated_urlconf_module(
                get_generated_urlconf_file(group_route.route.route_name))

            group_settings_cache['route_id'] = group_route.route.pk

        # Set Cache
        cache.set(cache_key, group_settings_cache)

    @cached_property
    def _validate(self):
        """
        :return:
        """
        errors = []

        try:
            dynamic_route_middleware_position = \
                settings.MIDDLEWARE_CLASSES.index(DynamicRouteMiddleware.name)
        except ValueError:
            errors.append(_("%s need to be "
                            "added into settings.MIDDLEWARE_CLASSES" % DynamicRouteMiddleware.name))
        else:
            if dynamic_route_middleware_position > settings.MIDDLEWARE_CLASSES.index(self.name):
                errors.append(_(DynamicRouteMiddleware.name + '\'s position should be before ' +
                                self.name))
        return True

    def _calculate_user_urlconf(self, request_path, url_conf):
        """
        :param request_path:
        :param url_conf:
        :return:
        """
        if isinstance(url_conf, six.string_types):
            return url_conf
        elif isinstance(url_conf, (list, tuple, set)):
            _temp = None
            for uc in url_conf:
                try:
                    resolver = get_resolver(uc)
                    resolver.resolve(request_path)
                    _temp = uc
                    break
                except Resolver404:
                    continue
            # @TODO: need some decision what should do if result is None
            return _temp
        else:
            return None

    def _calculate_anonymous_urlconf(self, request):
        """
        """
        cache = caches[getattr(settings, 'HACS_CACHE_SETTING_NAME', HACS_CACHE_SETTING_NAME)]
        cache_key = get_user_key(request)
        user_settings = cache.get(cache_key) or dict()
        try:
            return user_settings['settings']['urlconf']
        except KeyError:
            try:
                # The hacs convention anonymous user pk is 0
                user_route = ContentTypeRoutingTable.objects.get(
                    site=request.site,
                    content_type=ContentType.objects.get_for_model(UserModel),
                    object_id=0)
                generate_urlconf_file_on_demand(user_route.route)
                urlconf = get_generated_urlconf_module(get_generated_urlconf_file(user_route.route.route_name))
            except ContentTypeRoutingTable.DoesNotExist:
                urlconf = None

            if 'settings' not in user_settings.keys():
                user_settings['settings'] = dict()
            user_settings['settings']['urlconf'] = urlconf
            # Update Cache
            cache.set(cache_key, user_settings, 3600 * 24)

            return urlconf


__all__ = ("DynamicRouteMiddleware", "FirewallMiddleware", )
